@@ -33,6 +33,7 @@ x1_full[SparseArrays.nonzeroinds(spv_x1)] = nonzeros(spv_x1)
     @test SparseArrays.nonzeroinds(x) == [2, 5, 6]
     @test nonzeros(x) == [1.25, -0.75, 3.5]
     @test count(SparseVector(8, [2, 5, 6], [true,false,true])) == 2
+    @test count(SparseVector(8, [2, 5, 6], [true,false,true]), init=Int16(2))::Int16 == 4
     y = SparseVector(8, Int128[4], [5])
     @test y isa SparseVector{Int,Int128}
     @test @inferred size(y) == (@inferred(length(y))::Int128,)
@@ -58,6 +59,7 @@ end
         @test isa(Array(x), Vector{Float64})
         @test Array(x) == xf
         @test Vector(x) == xf
+        @test collect(x) == xf
     end
 end
 @testset "show" begin
@@ -66,7 +68,7 @@ end
     @test occursin("3.5", string(spv_x1))
 
     # issue #30589
-    @test repr("text/plain", sparse([true])) == "1-element SparseArrays.SparseVector{Bool, $Int} with 1 stored entry:\n  [1]  =  1"
+    @test repr("text/plain", sparse([true])) == "1-element $(SparseArrays.SparseVector){Bool, $Int} with 1 stored entry:\n  [1]  =  1"
 end
 
 ### Comparison helper to ensure exact equality with internal structure
@@ -215,6 +217,13 @@ end
             end
         end
     end
+
+    @testset "Undef initializer" begin
+        v = SparseVector{Float32, Int16}(undef, 4)
+        @test size(v) == (4, )
+        @test eltype(v) === Float32
+        @test v == spzeros(Float32, 4)
+    end
 end
 ### Element access
 
@@ -282,6 +291,10 @@ end
             @test Array(r) == Array(x)[bIv]
         end
     end
+    @testset "index with colon" begin
+        @test issparse(spzeros(0)[:])
+        @test isempty(spzeros(0)[:])
+    end
 end
 @testset "setindex" begin
     let xc = spzeros(Float64, 8)
@@ -340,6 +353,10 @@ end
         @test findall(!iszero, xc) == findall(!iszero, fc)
         @test findnz(xc) == ([2, 3, 5], [1.25, 0, -0.75])
     end
+    let Xc = spdiagm(spv_x1)
+        @test all(isempty, findnz(@view Xc[:,1]))
+        @test findnz(@view Xc[:,2]) == ([2], [1.25])
+    end
 end
 ### Array manipulation
 
@@ -447,7 +464,7 @@ end
         @test Vector(x1) == collect(x)
     end
 end
-@testset "vec/reinterpret/float/complex" begin
+@testset "vec/float/complex" begin
     a = SparseVector(8, [2, 5, 6], Int32[12, 35, 72])
     # vec
     @test vec(a) == a
@@ -567,6 +584,8 @@ end
             densevec = fill(1., N)
             densemat = fill(1., N, 1)
             diagmat = Diagonal(densevec)
+            # inferrability (https://github.com/JuliaSparse/SparseArrays.jl/pull/92)
+            cat_with_constdims(args...) = cat(args...; dims=(1,2))
             # Test that concatenations of pairwise combinations of sparse vectors with dense
             # vectors/matrices, sparse matrices, or special matrices yield sparse arrays
             for othervecormat in (densevec, densemat, spmat)
@@ -580,6 +599,9 @@ end
                 @test issparse(hvcat((2,), othervecormat, spvec))
                 @test issparse(cat(spvec, othervecormat; dims=(1,2)))
                 @test issparse(cat(othervecormat, spvec; dims=(1,2)))
+
+                @test issparse(@inferred cat_with_constdims(spvec, othervecormat))
+                @test issparse(@inferred cat_with_constdims(othervecormat, spvec))
             end
             # The preceding tests should cover multi-way combinations of those types, but for good
             # measure test a few multi-way combinations involving those types
@@ -591,6 +613,9 @@ end
             @test issparse(hvcat((5,), spvec, densemat, diagmat, densevec, spmat))
             @test issparse(cat(densemat, diagmat, spmat, densevec, spvec; dims=(1,2)))
             @test issparse(cat(spvec, diagmat, densevec, spmat, densemat; dims=(1,2)))
+
+            @test issparse(@inferred cat_with_constdims(densemat, diagmat, spmat, densevec, spvec))
+            @test issparse(@inferred cat_with_constdims(spvec, diagmat, densevec, spmat, densemat))
         end
         @testset "vertical concatenation of SparseVectors with different el- and ind-type (#22225)" begin
             spv6464 = SparseVector(0, Int64[], Int64[])
@@ -598,6 +623,12 @@ end
             @test isa(vcat(spv6464, SparseVector(0, Int32[], Int64[])), SparseVector{Int64,Int64})
             @test isa(vcat(spv6464, SparseVector(0, Int32[], Int32[])), SparseVector{Int64,Int64})
         end
+        @testset "horizontal concatenation of SparseVectors with different el- and ind-type (#22225)" begin
+            spv6464 = SparseVector(0, Int64[], Int64[])
+            @test isa(hcat(spv6464, SparseVector(0, Int64[], Int32[])), SparseMatrixCSC{Int64,Int64})
+            @test isa(hcat(spv6464, SparseVector(0, Int32[], Int64[])), SparseMatrixCSC{Int64,Int64})
+            @test isa(hcat(spv6464, SparseVector(0, Int32[], Int32[])), SparseMatrixCSC{Int64,Int64})
+        end
     end
 end
 @testset "sparsemat: combinations with sparse matrix" begin
@@ -917,7 +948,7 @@ end
         @test_throws ArgumentError findmin(x)
         @test_throws ArgumentError findmax(x)
     end
-    
+
     let v = spzeros(3) #Julia #44978
         v[1] = 2
         @test argmin(v) == 2
@@ -935,6 +966,32 @@ end
         v[3] = 2
         @test argmax(v) == 3
     end
+
+    let
+        v = spzeros(Bool, 5)
+        @test !any(v)
+        @test !all(v)
+        @test iszero(v)
+        @test count(v) == 0
+        v = SparseVector(5, [1], [false])
+        @test !any(v)
+        @test !all(v)
+        @test iszero(v)
+        @test count(v) == 0
+        v[2] = true
+        @test any(v)
+        @test !all(v)
+        @test count(v) == 1
+        v .= true
+        @test any(v)
+        @test all(v)
+        @test count(v) == length(v)
+    end
+
+    let v = spzeros(0)
+        @test all(!iszero, v)
+        @test !any(iszero, v)
+    end
 end
 
 ### linalg
@@ -1041,6 +1098,22 @@ end
                 @test isa(y, Vector{T})
                 @test y ≈ *(adjoint(A), xf)
             end
+
+            let A = randn(TA, 16, 16), x = sprand(Tx, 16, 0.7)
+                xf = Array(x)
+                for wrap in (M -> Symmetric(M, :U), M -> Symmetric(M, :L),
+                        M -> Hermitian(M, :U), M -> Hermitian(M, :L))
+                    for α in (0.0, 1.0, 2.0), β in (0.0, 0.5, 1.0)
+                        y = rand(T, 16)
+                        rr = α*wrap(A)*xf + β*y
+                        @test mul!(y, wrap(A), x, α, β) === y
+                        @test y ≈ rr
+                    end
+                    y = *(wrap(A), x)
+                    @test isa(y, Vector{T})
+                    @test y ≈ *(wrap(A), xf)
+                end
+            end
         end
     end
     @testset "sparse A * sparse x -> dense y" begin
@@ -1072,6 +1145,29 @@ end
             @test y ≈ *(transpose(Af), xf)
         end
 
+        let A = sprandn(16, 16, 0.5), x = sprand(16, 0.7)
+            Af = Array(A)
+            xf = Array(x)
+            for wrap in (M -> Symmetric(M, :U), M -> Symmetric(M, :L),
+                M -> Hermitian(M, :U), M -> Hermitian(M, :L),
+                M -> UpperTriangular(M), M -> UnitUpperTriangular(M),
+                M -> LowerTriangular(M), M -> UnitLowerTriangular(M),
+                M -> UpperTriangular(transpose(M)), M -> UnitUpperTriangular(transpose(M)),
+                M -> LowerTriangular(transpose(M)), M -> UnitLowerTriangular(transpose(M)),
+                M -> UpperTriangular(adjoint(M)), M -> UnitUpperTriangular(adjoint(M)),
+                M -> LowerTriangular(adjoint(M)), M -> UnitLowerTriangular(adjoint(M)),
+                M -> UpperTriangular(Symmetric(M)))
+                for α in (0.0, 1.0, 2.0), β in (0.0, 0.5, 1.0)
+                    y = rand(16)
+                    rr = α*wrap(Af)*xf + β*y
+                    @test mul!(y, wrap(A), x, α, β) === y
+                    @test y ≈ rr
+                end
+                y = wrap(A) * x
+                @test y ≈ *(wrap(Af), xf)
+            end
+        end
+
         let A = complex.(sprandn(7, 8, 0.5), sprandn(7, 8, 0.5)),
             x = complex.(sprandn(8, 0.6), sprandn(8, 0.6)),
             x2 = complex.(sprandn(7, 0.75), sprandn(7, 0.75))
@@ -1302,8 +1398,9 @@ end
     xdrop = copy(x)
     # This will keep index 1, 3, 4, 7 in xdrop
     f_drop(i, x) = (abs(x) == 1.) || (i in [1, 7])
-    SparseArrays.fkeep!(xdrop, f_drop)
+    SparseArrays.fkeep!(f_drop, xdrop)
     @test exact_equal(xdrop, SparseVector(7, [1, 3, 4, 7], [3., -1., 1., 3.]))
+    @test_deprecated SparseArrays.fkeep!(xdrop, f_drop)
 end
 
 @testset "dropzeros[!] with length=$m" for m in (10, 20, 30)
@@ -1433,17 +1530,48 @@ mutable struct t20488 end
 @testset "show" begin
     io = IOBuffer()
     show(io, MIME"text/plain"(), sparsevec(Int64[1], [1.0]))
-    @test String(take!(io)) == "1-element SparseArrays.SparseVector{Float64, Int64} with 1 stored entry:\n  [1]  =  1.0"
+    @test String(take!(io)) == "1-element $(SparseArrays.SparseVector){Float64, Int64} with 1 stored entry:\n  [1]  =  1.0"
     show(io, MIME"text/plain"(),  spzeros(Float64, Int64, 2))
-    @test String(take!(io)) == "2-element SparseArrays.SparseVector{Float64, Int64} with 0 stored entries"
+    @test String(take!(io)) == "2-element $(SparseArrays.SparseVector){Float64, Int64} with 0 stored entries"
     show(io, similar(sparsevec(rand(3) .+ 0.1), t20488))
     @test String(take!(io)) == "  [1]  =  #undef\n  [2]  =  #undef\n  [3]  =  #undef"
+    # Test that we don't introduce unnecessary padding for long sparse arrays
+    show(io, MIME"text/plain"(), SparseVector(div(typemax(Int32), 2), Int32[1], Int32[1]))
+    @test String(take!(io)) == "1073741823-element $(SparseArrays.SparseVector){Int32, Int32} with 1 stored entry:\n  [1]  =  1"
 end
 
 @testset "spzeros with index type" begin
     @test typeof(spzeros(Float32, Int16, 3)) == SparseVector{Float32,Int16}
 end
 
+@testset "binary operations on sparse vectors with union eltype" begin
+    A = SparseVector(2, [1,2], Union{Int, Missing}[1, missing])
+    for fun in (+, -, *, min, max)
+        if fun in (+, -)
+            @test collect(skipmissing(Array(fun(A, A)))) == collect(skipmissing(Array(fun(Array(A), Array(A)))))
+        end
+        @test collect(skipmissing(Array(map(fun, A, A)))) == collect(skipmissing(map(fun, Array(A), Array(A))))
+        @test collect(skipmissing(Array(broadcast(fun, A, A)))) == collect(skipmissing(broadcast(fun, Array(A), Array(A))))
+    end
+    b = convert(SparseVector{Union{Float64, Missing}}, sprandn(Float64, 10, 0.2)); b[rand(1:10, 3)] .= missing
+    C = convert(SparseVector{Union{Float64, Missing}}, sprandn(Float64, 10, 0.9)); C[rand(1:10, 3)] .= missing
+    CA = Array(C)
+    D = convert(SparseVector{Union{Float64, Missing}}, spzeros(Float64, 10)); D[rand(1:10, 3)] .= missing
+    E = convert(SparseVector{Union{Float64, Missing}}, spzeros(Float64, 10))
+    for B in (b, C, D, E), fun in (+, -, *, min, max)
+        BA = Array(B)
+        # reverse order for opposite nonzeroinds-structure
+        if fun in (+, -)
+            @test collect(skipmissing(Array(fun(B, C)))) == collect(skipmissing(Array(fun(BA, CA))))
+            @test collect(skipmissing(Array(fun(C, B)))) == collect(skipmissing(Array(fun(CA, BA))))
+        end
+        @test collect(skipmissing(Array(map(fun, B, C)))) == collect(skipmissing(map(fun, BA, CA)))
+        @test collect(skipmissing(Array(map(fun, C, B)))) == collect(skipmissing(map(fun, CA, BA)))
+        @test collect(skipmissing(Array(broadcast(fun, B, C)))) == collect(skipmissing(broadcast(fun, BA, CA)))
+        @test collect(skipmissing(Array(broadcast(fun, C, B)))) == collect(skipmissing(broadcast(fun, CA, BA)))
+    end
+end
+
 @testset "corner cases of broadcast arithmetic operations with scalars (#21515)" begin
     # test both scalar literals and variables
     areequal(a, b, c) = isequal(a, b) && isequal(b, c)
@@ -1518,6 +1646,12 @@ end
     @test length(nonzeros(simA)) == 0
 end
 
+@testset "map preserves index types" begin
+    v1 = spzeros(Float32, Int16, 10)
+    v2 = spzeros(Float32, Int32, 10)
+    @test eltype(typeof(SparseArrays.nonzeroinds(map(max, v1, v2)))) == Int32
+end
+
 @testset "Fast operations on full column views" begin
     n = 1000
     A = sprandn(n, n, 0.01)
